import 'package:film_gyaan/core.dart';

final _logger = injectLogger('film_gyaan.dx.movies');

/// A mixin for Movies which will be handling all the request made related
/// to movies
mixin MoviesDx on Core {
  /// Get popular movies.
  ///
  /// Get a list of the current popular movies on TMDB.
  /// This list updates daily.
  ///
  /// [page] : Specify which page to query.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-popular-movies).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MoviesList> getPopularMovies({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = PopularMoviesParams(
      page: page,
    );

    _logger.info('Getting popular movies for page $page');

    return defaultFlow<PopularMoviesParams, MoviesList>(
      core: this,
      params: params,
      serializer: (dynamic json) => MoviesList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get now playing movies.
  ///
  /// Get a list of movies in theatres. This is a release type query that looks
  /// for all movies that have a release type of 2 or 3 within the
  /// specified date range.
  ///
  /// You can optionally specify a region parameter which will narrow the
  /// search to only look for theatrical release dates within the
  /// specified country.
  ///
  /// [page] : Specify which page to query.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-now-playing).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MoviesList> getNowPlaying({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = NowPlayingMoviesParams(
      page: page,
    );

    _logger.info('Getting now playing movies for page $page');

    return defaultFlow<NowPlayingMoviesParams, MoviesList>(
      core: this,
      params: params,
      serializer: (dynamic json) => MoviesList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get upcoming movies.
  ///
  /// Get a list of upcoming movies in theatres. This is a release type query
  /// that looks for all movies that have a release type of 2 or 3 within
  /// the specified date range.
  ///
  /// You can optionally specify a region parameter which will narrow the
  /// search to only look for theatrical release dates within the
  /// specified country.
  ///
  /// [page] : Specify which page to query.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-upcoming).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MoviesList> getUpcomingMovies({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = UpcomingMoviesParams(
      page: page,
    );

    _logger.info('Getting upcoming movies for page $page');

    return defaultFlow<UpcomingMoviesParams, MoviesList>(
      core: this,
      params: params,
      serializer: (dynamic json) => MoviesList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get top-rated movies.
  ///
  /// Get the top rated movies on TMDB.
  ///
  /// [page] : Specify which page to query.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-top-rated-movies).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MoviesList> getTopRatedMovies({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = TopRatedMoviesParams(
      page: page,
    );

    _logger.info('Getting top rated movies for page $page');

    return defaultFlow<TopRatedMoviesParams, MoviesList>(
      core: this,
      params: params,
      serializer: (dynamic json) => MoviesList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get latest movie.
  ///
  /// Get the most newly created movie. This is a live response and
  /// will continuously change.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-latest-movie).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MovieDetails> getLatestMovie() async {
    var params = LatestMovieParams();

    _logger.info('Getting a latest movie');

    return defaultFlow<LatestMovieParams, MovieDetails>(
      core: this,
      params: params,
      serializer: (dynamic json) => MovieDetails.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie details.
  ///
  /// Get the primary information about a movie.
  ///
  /// Will also be giving videos, images, credits, reviews, recommendations,
  /// similar and external_ids in the response. So can use it from the
  /// same API, instead of calling different APIs again.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-details).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MovieDetails> getMovieDetails({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = MovieInformationParams(
      movieId: movieId,
    );

    _logger.info('Getting details for $movieId movie');

    return defaultFlow<MovieInformationParams, MovieDetails>(
      core: this,
      params: params,
      serializer: (dynamic json) => MovieDetails.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get alternative movies titles.
  ///
  /// Get all of the alternative titles for a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-alternative-titles).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<AlternativeTitles> getMovieAlternativeTitles({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = AlternativeTitlesParams(
      movieId: movieId,
    );

    _logger.info('Getting alternative titles for $movieId movie');

    return defaultFlow<AlternativeTitlesParams, AlternativeTitles>(
      core: this,
      params: params,
      serializer: (dynamic json) => AlternativeTitles.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie credits.
  ///
  /// Get the cast and crew for a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-credits).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Credits> getMovieCredits({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = CreditsParams(
      movieId: movieId,
    );

    _logger.info('Getting credits for $movieId movie');

    return defaultFlow<CreditsParams, Credits>(
      core: this,
      params: params,
      serializer: (dynamic json) => Credits.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie external ids.
  ///
  /// Get the external ids for a movie. We currently support the following external sources.
  ///
  /// Media Databases	: IMDb ID
  ///
  /// Social IDs : Facebook, Instagram, Twitter
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-external-ids).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<ExternalIds> getMovieExternalIds({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = ExternalIdsParams(
      movieId: movieId,
    );

    _logger.info('Getting credits for $movieId movie');

    return defaultFlow<ExternalIdsParams, ExternalIds>(
      core: this,
      params: params,
      serializer: (dynamic json) => ExternalIds.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie images.
  ///
  /// Get the images that belong to a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-images).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Images> getMovieImages({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = ImagesParams(
      movieId: movieId,
    );

    _logger.info('Getting images for $movieId movie');

    return defaultFlow<ImagesParams, Images>(
      core: this,
      params: params,
      serializer: (dynamic json) => Images.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie keywords.
  ///
  /// Get the keywords that have been added to a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-keywords).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Keywords> getMovieKeywords({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = KeywordsParams(
      movieId: movieId,
    );

    _logger.info('Getting keywords for $movieId movie');

    return defaultFlow<KeywordsParams, Keywords>(
      core: this,
      params: params,
      serializer: (dynamic json) => Keywords.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie lists.
  ///
  /// Get a list of lists that this movie belongs to.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-lists).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MovieLists> getMovieLists({
    required int movieId,
    int page = 1,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = MovieListsParams(
      movieId: movieId,
      page: page,
    );

    _logger.info('Getting lists for $movieId movie');

    return defaultFlow<MovieListsParams, MovieLists>(
      core: this,
      params: params,
      serializer: (dynamic json) => MovieLists.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie recommendations.
  ///
  /// Get a list of recommended movies for a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-recommendations).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MoviesList> getMovieRecommendations({
    required int movieId,
    int page = 1,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = RecommendationsParams(
      movieId: movieId,
      page: page,
    );

    _logger.info('Getting recommendations for $movieId movie');

    return defaultFlow<RecommendationsParams, MoviesList>(
      core: this,
      params: params,
      serializer: (dynamic json) => MoviesList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie release dates.
  ///
  /// Get the release date along with the certification for a movie.
  ///
  /// Release dates support different types:
  ///
  /// 1. Premiere
  /// 2. Theatrical (limited)
  /// 3. Theatrical
  /// 4. Digital
  /// 5. Physical
  /// 6. TV
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-release-dates).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<ReleaseDates> getMovieReleaseDates({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = ReleaseDatesParams(
      movieId: movieId,
    );

    _logger.info('Getting release for $movieId movie');

    return defaultFlow<ReleaseDatesParams, ReleaseDates>(
      core: this,
      params: params,
      serializer: (dynamic json) => ReleaseDates.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie reviews.
  ///
  /// Get the user reviews for a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-reviews).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<ReviewsList> getMovieReviews({
    required int movieId,
    int page = 1,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = ReviewsParams(
      movieId: movieId,
      page: page,
    );

    _logger.info('Getting reviews for $movieId movie');

    return defaultFlow<ReviewsParams, ReviewsList>(
      core: this,
      params: params,
      serializer: (dynamic json) => ReviewsList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get similar movies.
  ///
  /// Get a list of similar movies. This is not the same as the "Recommendation" system you see on the website.
  ///
  /// These items are assembled by looking at keywords and genres.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-similar-movies).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<MoviesList> getSimilarMovies({
    required int movieId,
    int page = 1,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = SimilarParams(
      movieId: movieId,
      page: page,
    );

    _logger.info('Getting similar movies for $movieId movie');

    return defaultFlow<SimilarParams, MoviesList>(
      core: this,
      params: params,
      serializer: (dynamic json) => MoviesList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie translations.
  ///
  /// Get a list of translations that have been created for a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-translations).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Translations> getMovieTranslations({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = TranslationsParams(
      movieId: movieId,
    );

    _logger.info('Getting translations for $movieId movie');

    return defaultFlow<TranslationsParams, Translations>(
      core: this,
      params: params,
      serializer: (dynamic json) => Translations.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get movie videos.
  ///
  /// Get the videos that have been added to a movie.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/movies/get-movie-videos).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Videos> getMovieVideos({
    required int movieId,
  }) async {
    Ensure(movieId > 0).isTrue('Movie Id should be > 0');

    var params = VideosParams(
      movieId: movieId,
    );

    _logger.info('Getting videos for $movieId movie');

    return defaultFlow<VideosParams, Videos>(
      core: this,
      params: params,
      serializer: (dynamic json) => Videos.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }
}
