import 'package:film_gyaan/core.dart';

final _logger = injectLogger('film_gyaan.dx.people');

/// A mixin for People which will be handling all the request made related
/// to people
mixin PeopleDx on Core {
  /// Get popular people.
  ///
  /// Get a list of the current popular people on TMDB.
  /// This list updates daily.
  ///
  /// [page] : Specify which page to query.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-popular-people).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<PeopleList> getPopularPeople({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = PopularPersonParams(
      page: page,
    );

    _logger.info('Getting popular people for page $page');

    return defaultFlow<PopularPersonParams, PeopleList>(
      core: this,
      params: params,
      serializer: (dynamic json) => PeopleList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get latest person.
  ///
  /// Get the most newly created person. This is a live response and
  /// will continuously change.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-latest-person).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<PeopleDetails> getLatestPerson() async {
    var params = PeopleLatestParams();

    _logger.info('Getting a latest movie');

    return defaultFlow<PeopleLatestParams, PeopleDetails>(
      core: this,
      params: params,
      serializer: (dynamic json) => PeopleDetails.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get person details.
  ///
  /// Get the primary person details by id.
  ///
  /// Details like combined_credits, external_ids, images and tagged_images
  /// will also be added.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-person-details).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<PeopleDetails> getPersonDetails({
    required int personId,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');

    var params = PeopleDetailsParams(
      personId: personId,
    );

    _logger.info('Getting details for $personId person');

    return defaultFlow<PeopleDetailsParams, PeopleDetails>(
      core: this,
      params: params,
      serializer: (dynamic json) => PeopleDetails.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get Movie Credits.
  ///
  /// Get the movie credits for a person.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-person-movie-credits).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Credits> getPersonMovieCredits({
    required int personId,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');

    var params = PeopleMovieCreditsParams(
      personId: personId,
    );

    _logger.info('Getting movie credits for $personId person');

    return defaultFlow<PeopleMovieCreditsParams, Credits>(
      core: this,
      params: params,
      serializer: (dynamic json) => Credits.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV Credits.
  ///
  /// Get the TV show credits for a person.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-person-tv-credits).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Credits> getPersonTvCredits({
    required int personId,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');

    var params = PeopleTvCreditsParams(
      personId: personId,
    );

    _logger.info('Getting tv credits for $personId person');

    return defaultFlow<PeopleTvCreditsParams, Credits>(
      core: this,
      params: params,
      serializer: (dynamic json) => Credits.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get Combined Credits.
  ///
  /// Get the movie and TV credits together in a single response.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-person-combined-credits).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Credits> getPersonCombinedCredits({
    required int personId,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');

    var params = PeopleMovieTvCreditsParams(
      personId: personId,
    );

    _logger.info('Getting combined credits for $personId person');

    return defaultFlow<PeopleMovieTvCreditsParams, Credits>(
      core: this,
      params: params,
      serializer: (dynamic json) => Credits.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get External IDs
  ///
  /// Get the external ids for a person. We currently support the
  /// following external sources.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-person-external-ids).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<ExternalIds> getPersonExternalId({
    required int personId,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');

    var params = PeopleExternalIdParams(
      personId: personId,
    );

    _logger.info('Getting external ids for $personId person');

    return defaultFlow<PeopleExternalIdParams, ExternalIds>(
      core: this,
      params: params,
      serializer: (dynamic json) => ExternalIds.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get Images
  ///
  /// Get the images for a person.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-person-images).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<PeopleImages> getPersonImages({
    required int personId,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');

    var params = PeopleImagesParams(
      personId: personId,
    );

    _logger.info('Getting external ids for $personId person');

    return defaultFlow<PeopleImagesParams, PeopleImages>(
      core: this,
      params: params,
      serializer: (dynamic json) => PeopleImages.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get tagged images
  ///
  /// Get the images that this person has been tagged in.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-tagged-images).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<PeopleTaggedImages> getPersonTaggedImages({
    required int personId,
    int page = 1,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = PeopleTaggedImagesParams(
      personId: personId,
      page: page,
    );

    _logger.info('Getting tagged images for $personId person');

    return defaultFlow<PeopleTaggedImagesParams, PeopleTaggedImages>(
      core: this,
      params: params,
      serializer: (dynamic json) => PeopleTaggedImages.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get Translations
  ///
  /// Get a list of translations that have been created for a person.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/people/get-person-translations).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Translations> getPersonTranslations({
    required int personId,
  }) async {
    Ensure(personId > 0).isTrue('Person Id should be > 0');

    var params = PeopleTranslationsParams(
      personId: personId,
    );

    _logger.info('Getting tagged images for $personId person');

    return defaultFlow<PeopleTranslationsParams, Translations>(
      core: this,
      params: params,
      serializer: (dynamic json) => Translations.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }
}
