import 'package:film_gyaan/core.dart';

final _logger = injectLogger('film_gyaan.dx.tv');

/// A mixin for People which will be handling all the request made related
/// to tv
mixin TvDx on Core {
  /// Get TV Airing Today
  ///
  /// Get a list of TV shows that are airing today. This query is purely
  /// day based as we do not currently support airing times.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-tv-airing-today).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<TvsList> getTvAiringToday({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = TvAiringTodayParams(
      page: page,
    );

    _logger.info('Getting tv airing today for page $page');

    return defaultFlow<TvAiringTodayParams, TvsList>(
      core: this,
      params: params,
      serializer: (dynamic json) => TvsList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV On The Air
  ///
  /// Get a list of shows that are currently on the air.
  ///
  /// This query looks for any TV show that has an episode with an air date
  /// in the next 7 days.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-tv-on-the-air).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<TvsList> getTvOnTheAir({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = TvOnAirParams(
      page: page,
    );

    _logger.info('Getting tv on the air for page $page');

    return defaultFlow<TvOnAirParams, TvsList>(
      core: this,
      params: params,
      serializer: (dynamic json) => TvsList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV Popular
  ///
  /// Get a list of the current popular TV shows on TMDB. This
  /// list updates daily.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-popular-tv-shows).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<TvsList> getTvPopular({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = TvPopularParams(
      page: page,
    );

    _logger.info('Getting tv popular for page $page');

    return defaultFlow<TvPopularParams, TvsList>(
      core: this,
      params: params,
      serializer: (dynamic json) => TvsList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV Top Rated
  ///
  /// Get a list of the top rated TV shows on TMDB.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-top-rated-tv).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<TvsList> getTvTopRated({
    int page = 1,
  }) async {
    Ensure(page >= 1).isTrue('Page should\'nt be less than 1');
    Ensure(page <= 1000).isTrue('Page should\'nt be more than 1000');

    var params = TvTopRatedParams(
      page: page,
    );

    _logger.info('Getting tv top rated for page $page');

    return defaultFlow<TvTopRatedParams, TvsList>(
      core: this,
      params: params,
      serializer: (dynamic json) => TvsList.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV Latest
  ///
  /// Get the most newly created TV show. This is a live
  /// response and will continuously change.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-latest-tv).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<TvDetails> getTvLatest() async {
    var params = TvLatestParams();

    _logger.info('Getting tv latest');

    return defaultFlow<TvLatestParams, TvDetails>(
      core: this,
      params: params,
      serializer: (dynamic json) => TvDetails.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV Details
  ///
  /// Get the primary TV show details by id.
  ///
  /// Other details like account_states, aggregate_credits, alternative_titles,
  /// content_ratings, credits, episode_groups, external_ids, images,
  /// keywords, recommendations, reviews, screened_theatrically, similar,
  /// translations, and videos.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-tv-details).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<TvDetails> getTvDetails({
    required int tvId,
  }) async {
    Ensure(tvId > 0).isTrue('Tv Id should be > 0');

    var params = TvDetailsParams(
      tvId: tvId,
    );

    _logger.info('Getting tv details for $tvId');

    return defaultFlow<TvDetailsParams, TvDetails>(
      core: this,
      params: params,
      serializer: (dynamic json) => TvDetails.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV aggregate credits
  ///
  /// Get the aggregate credits (cast and crew) that have been added
  /// to a TV show.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-tv-aggregate-credits).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<Credits> getTvAggregateCredits({
    required int tvId,
  }) async {
    Ensure(tvId > 0).isTrue('Tv Id should be > 0');

    var params = TvAggregateCreditsParams(
      tvId: tvId,
    );

    _logger.info('Getting tv aggregated credits for $tvId');

    return defaultFlow<TvAggregateCreditsParams, Credits>(
      core: this,
      params: params,
      serializer: (dynamic json) => Credits.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV alternative titles
  ///
  /// Returns all of the alternative titles for a TV show.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-tv-alternative-titles).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<AlternativeTitles> getTvAlternativeTitles({
    required int tvId,
  }) async {
    Ensure(tvId > 0).isTrue('Tv Id should be > 0');

    var params = TvAlternativeTitlesParams(
      tvId: tvId,
    );

    _logger.info('Getting tv alternative titles for $tvId');

    return defaultFlow<TvAlternativeTitlesParams, AlternativeTitles>(
      core: this,
      params: params,
      serializer: (dynamic json) => AlternativeTitles.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }

  /// Get TV content ratings
  ///
  /// Get the list of content ratings (certifications)
  /// that have been added to a TV show.
  ///
  /// For more details on the API go [here](https://developers.themoviedb.org/3/tv/get-tv-content-ratings).
  ///
  /// Throws [FilmGyaanException] on an error.
  Future<ContentRatings> getTvContentRatings({
    required int tvId,
  }) async {
    Ensure(tvId > 0).isTrue('Tv Id should be > 0');

    var params = TvContentRatingsParams(
      tvId: tvId,
    );

    _logger.info('Getting tv content ratings for $tvId');

    return defaultFlow<TvContentRatingsParams, ContentRatings>(
      core: this,
      params: params,
      serializer: (dynamic json) => ContentRatings.fromMap(
        json as Map<String, dynamic>,
      ),
    );
  }
}
